"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _app = _interopRequireDefault(require("firebase/app"));

require("firebase/auth");

var _methods = _interopRequireDefault(require("./methods"));

var _adminOnRest = require("admin-on-rest");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

/**
 * @param {string[]|Object[]} trackedResources Array of resource names or array of Objects containing name and
 * optional path properties (path defaults to name)
 * @param {Object} firebaseConfig Options Firebase configuration
 */
var BaseConfiguration = {
  initialQueryTimeout: 1000,
  timestampFieldNames: {
    createdAt: "createdAt",
    updatedAt: "updatedAt"
  },
  firebasePersistence: _app.default.auth.Auth.Persistence.SESSION
};
var resourcesStatus = {};
var resourcesReferences = {};
var resourcesData = {};
var resourcesPaths = {};
var resourcesUploadFields = {};
var globalMethods = {};

var initializeResource =
/*#__PURE__*/
function () {
  var _ref2 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee(_ref, initialQueryTimeout) {
    var name, isPublic, ref;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            name = _ref.name, isPublic = _ref.isPublic;
            ref = resourcesReferences[name] = _app.default.database().ref(resourcesPaths[name]);
            resourcesData[name] = [];

            if (!isPublic) {
              _context.next = 8;
              break;
            }

            _context.next = 6;
            return subscribeResource(ref, name);

          case 6:
            _context.next = 9;
            break;

          case 8:
            _app.default.auth().onAuthStateChanged(function (auth) {
              if (auth) {
                subscribeResource(ref, name);
              }
            });

          case 9:
            _context.next = 11;
            return new Promise(function (r) {
              return setTimeout(r, initialQueryTimeout);
            });

          case 11:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function initializeResource(_x, _x2) {
    return _ref2.apply(this, arguments);
  };
}();

var sanitizeResource = function sanitizeResource(resource) {
  if (typeof resource === "string") {
    resource = {
      name: resource
    };
  }

  var _resource = resource,
      name = _resource.name,
      path = _resource.path,
      uploadFields = _resource.uploadFields;

  if (!resource.name) {
    throw new Error("name is missing from resource ".concat(resource));
  }

  resourcesUploadFields[name] = uploadFields || [];
  resourcesPaths[name] = path || "/" + name;
  resourcesData[name] = {};
  return resource;
};

var subscribeResource =
/*#__PURE__*/
function () {
  var _ref3 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee2(ref, name) {
    var snapshot, entries, lastId;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return ref.orderByKey().once("value");

          case 2:
            snapshot = _context2.sent;
            entries = snapshot.val() || {};
            lastId = "";
            Object.keys(entries).map(function (id) {
              resourcesData[name][id] = globalMethods.postRead(_objectSpread({}, entries[id], {
                id: id
              }), id, name);
              lastId = id;
            });
            ref.orderByKey().startAt(lastId).on("child_added", function (childSnapshot) {
              var id = childSnapshot.key;
              resourcesData[name][id] = globalMethods.postRead(_objectSpread({}, childSnapshot.val(), {
                id: id
              }), id, name);
            });
            ref.on("child_changed", function (childSnapshot) {
              var id = childSnapshot.key;
              resourcesData[name][id] = globalMethods.postRead(_objectSpread({}, childSnapshot.val(), {
                id: id
              }), id, name);
            });
            ref.on("child_removed", function (oldChildSnapshot) {
              var id = oldChildSnapshot.key;
              delete resourcesData[name][id];
            });

          case 9:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function subscribeResource(_x3, _x4) {
    return _ref3.apply(this, arguments);
  };
}();

var _default = function _default() {
  var firebaseConfig = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var _ref4 = arguments.length > 1 ? arguments[1] : undefined,
      trackedResources = _ref4.trackedResources,
      options = _ref4.options;

  options = Object.assign({}, BaseConfiguration, options || {});
  var _options = options,
      timestampFieldNames = _options.timestampFieldNames,
      initialQueryTimeout = _options.initialQueryTimeout;

  if (_app.default.apps.length === 0) {
    _app.default.initializeApp(firebaseConfig);

    _app.default.auth().setPersistence(options.firebasePersistence);
  }
  /* InternalMethods */


  var methods = Object.assign({}, _methods.default, options.methods || {});
  globalMethods.preSave = methods.preSave;
  globalMethods.postRead = methods.postRead; // Sanitize Resources

  trackedResources.map(function (resource) {
    resource = sanitizeResource(resource);
    resourcesStatus[resource.name] = initializeResource(resource, initialQueryTimeout);
  });
  /**
   * @param {string} type Request type, e.g GET_LIST
   * @param {string} resourceName Resource name, e.g. "posts"
   * @param {Object} payload Request parameters. Depends on the request type
   * @returns {Promise} the Promise for a REST response
   */

  return (
    /*#__PURE__*/
    function () {
      var _ref5 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee3(type, resourceName, params) {
        var result;
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return resourcesStatus[resourceName];

              case 2:
                result = null;
                _context3.t0 = type;
                _context3.next = _context3.t0 === _adminOnRest.GET_LIST ? 6 : _context3.t0 === _adminOnRest.GET_MANY ? 6 : _context3.t0 === _adminOnRest.GET_MANY_REFERENCE ? 6 : _context3.t0 === _adminOnRest.GET_ONE ? 10 : _context3.t0 === _adminOnRest.DELETE ? 11 : _context3.t0 === _adminOnRest.UPDATE ? 15 : _context3.t0 === _adminOnRest.CREATE ? 15 : 20;
                break;

              case 6:
                _context3.next = 8;
                return methods.getMany(params, resourcesData[resourceName], type);

              case 8:
                result = _context3.sent;
                return _context3.abrupt("return", result);

              case 10:
                return _context3.abrupt("return", methods.getOne(params, resourcesData[resourceName]));

              case 11:
                _context3.next = 13;
                return methods.del(params, resourcesPaths[resourceName], resourcesUploadFields[resourceName] || []);

              case 13:
                result = _context3.sent;
                return _context3.abrupt("return", result);

              case 15:
                params.__id = methods.getId(params, resourcesData[resourceName], type, resourcesPaths[resourceName]);
                _context3.next = 18;
                return methods.save(params, resourceName, type, resourcesPaths[resourceName], globalMethods.preSave, resourcesUploadFields[resourceName].map(function (field) {
                  return methods.upload(field, params, resourceName, resourcesPaths[resourceName]);
                }), timestampFieldNames);

              case 18:
                result = _context3.sent;
                return _context3.abrupt("return", result);

              case 20:
                // eslint-disable-next-line no-console
                console.error("Undocumented method: ", type);
                return _context3.abrupt("return", {
                  data: []
                });

              case 22:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      return function (_x5, _x6, _x7) {
        return _ref5.apply(this, arguments);
      };
    }()
  );
};

exports.default = _default;